"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const inquirer_1 = require("inquirer");
const util_1 = require("../helpers/util");
const config_1 = require("../config");
const create = async (projectName) => {
    const rootDir = path_1.resolve(__dirname, "..", "..");
    if (!projectName)
        return console.log("project name is not provided!");
    try {
        const result = await inquirer_1.prompt([
            {
                name: "ProjectSetup",
                type: "checkbox",
                message: "Project setup",
                choices: ["typescript", "scss", "eslint"]
            }
        ]);
        let template = "js";
        if (result.ProjectSetup.includes("typescript"))
            template = "ts";
        const projectPath = path_1.resolve(".", `${projectName}`);
        const ext = template === "js" ? "js" : "tsx";
        console.log("Creating project...");
        util_1.copyDir(path_1.resolve(rootDir, "templates", "common"), projectPath);
        util_1.copyDir(path_1.resolve(rootDir, "templates", template), projectPath);
        const indexPath = path_1.resolve(projectPath, "src", "index");
        const appPath = path_1.resolve(projectPath, "src", "components", "App");
        if (template === "ts") {
            fs_1.renameSync(`${indexPath}.js`, `${indexPath}.${ext}`);
            fs_1.renameSync(`${appPath}.js`, `${appPath}.${ext}`);
        }
        const packageJSON = JSON.parse(fs_1.readFileSync(path_1.resolve(projectPath, "package.json"), "utf-8"));
        const webpackConfigPath = path_1.resolve(projectPath, `webpack.config.${template}`);
        let webpackConfigText = fs_1.readFileSync(webpackConfigPath, "utf-8");
        if (!result.ProjectSetup.includes("scss")) {
            // remove scss related packages
            config_1.scssRelatedPackages.forEach(el => {
                delete packageJSON.devDependencies[el];
                delete packageJSON.dependencies[el];
            });
            // rename main.scss to main.css
            const mainStylesPath = path_1.resolve(projectPath, "src", "styles", "main");
            fs_1.renameSync(`${mainStylesPath}.scss`, `${mainStylesPath}.css`);
            // change main styles import extension in index file
            const indexPath = path_1.resolve(projectPath, "src", `index.${ext}`);
            const newIndexText = fs_1.readFileSync(indexPath, "utf-8").replace("main.scss", "main.css");
            fs_1.writeFileSync(indexPath, newIndexText);
            // remove scss related configs from webpack.config.{js|ts}
            config_1.scssRelatedWebpackConfigs.forEach(([src, target]) => {
                webpackConfigText = webpackConfigText.replace(src, target);
            });
        }
        if (!result.ProjectSetup.includes("eslint")) {
            // eslint packages to remove
            config_1.eslintRelatedPackages.forEach(el => {
                delete packageJSON.devDependencies[el];
                delete packageJSON.dependencies[el];
            });
            // remove eslint related files
            config_1.eslintRelatedFiles.forEach(el => fs_1.unlinkSync(path_1.join(projectName, el)));
            // remove eslint related scripts
            config_1.eslintRelatedScripts.forEach(el => delete packageJSON.scripts[el]);
            // remove eslint related configs webpack.config.{js|ts}
            config_1.eslintRelatedWebpackConfigs.forEach(([src, target]) => {
                webpackConfigText = webpackConfigText.replace(src, target);
            });
        }
        fs_1.writeFileSync(webpackConfigPath, webpackConfigText);
        const dependencies = Object.keys(packageJSON.dependencies);
        const devDependencies = Object.keys(packageJSON.devDependencies);
        packageJSON.name = `${projectName}`;
        delete packageJSON.dependencies;
        delete packageJSON.devDependencies;
        fs_1.writeFileSync(path_1.resolve(projectPath, "package.json"), JSON.stringify(packageJSON, null, 2));
        console.log("Installing dependencies...");
        await util_1.exec(`npm install ${dependencies.join("@latest ")}`, { cdw: projectPath });
        console.log("Installing dev dependencies...");
        await util_1.exec(`npm install -D ${devDependencies.join("@latest ")}`, { cdw: projectPath });
        console.log("Done!");
        console.log(`To start dev server run: \ncd ${projectName}\nnpm run dev`);
    }
    catch (e) {
        console.error(e);
    }
};
exports.default = create;
